//-----------------------------------------------------------------------
// <copyright file="Input.WixlibTests.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
// <summary>
//     Test for giving wixlib files as input to Light
// </summary>
//-----------------------------------------------------------------------

namespace Microsoft.Tools.WindowsInstallerXml.Test.Tests.Tools.Light.Input
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text;

    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Microsoft.Tools.WindowsInstallerXml.Test;

    /// <summary>
    /// Test for giving wixlib files as input to Light
    /// </summary>
    [TestClass]
    public class WixlibTests
    {
        private static readonly string TestDataDirectory = Environment.ExpandEnvironmentVariables(@"%WIX%\test\data\Tools\Light\Input\WixlibTests");

        [TestMethod]
        [Description("Verify that Light can link a wixlib")]
        [Priority(1)]
        public void SingleWixlib()
        {
            Candle candle = new Candle();
            candle.SourceFiles.Add(Path.Combine(WixTests.SharedAuthoringDirectory, "BasicProduct.wxs"));
            candle.Run();

            // Build a wixlib
            // Lit lit = new Lit();
            // lit.Run();

            Light light = new Light();
            light.ObjectFiles.Add(Path.Combine(WixlibTests.TestDataDirectory, @"SingleWixlib\BasicProduct.wixlib"));
            light.OutputFile = Path.Combine(candle.OutputDirectory, "BasicProduct.msi");
            light.Run();

            Verifier.VerifyResults(Path.Combine(WixlibTests.TestDataDirectory, @"SingleWixlib\expected.msi"), light.OutputFile);
        }

        [TestMethod]
        [Description("Verify that Light can link multiple wixlibs")]
        [Priority(2)]
        [Ignore]
        public void MultipleWixlibs()
        {
        }

    }
}
